@@ -259,6 +259,8 @@ let unspec d = API.ContextualConversion.(!<(unspecC (!> d)))
 
 let core_builtins = let open BuiltIn in let open ContextualConversion in [
 
+  LPDoc "File generated by elpi -document-builtins, do not edit";
+
   LPDoc " == Core builtins =====================================";
 
   LPDoc " -- Logic --";
@@ -270,12 +272,15 @@ let core_builtins = let open BuiltIn in let open ContextualConversion in [
   LPCode "pred false.";
 
   LPCode "external pred (=) o:A, o:A. % unification";
+
+  LPCode "external pred (pi) i:A -> prop.";
+  LPCode "external pred (sigma) i:A -> prop.";
   
   MLData BuiltInData.int;
   MLData BuiltInData.string;
   MLData BuiltInData.float;
 
-  LPCode "pred (;) o:prop, o:prop.";
+  LPCode "pred (;) i:prop, i:prop.";
   LPCode "(A ; _) :- A.";
   LPCode "(_ ; B) :- B.";
 
@@ -286,6 +291,8 @@ let core_builtins = let open BuiltIn in let open ContextualConversion in [
   LPCode "type (as) A -> A -> A.";
   LPCode "type (=>) prop -> prop -> prop.";
   LPCode "type (=>) list prop -> prop -> prop.";
+  LPCode "type (==>) prop -> prop -> prop."; (* not really needed since the parser emits a => *)
+  LPCode "type (==>) list prop -> prop -> prop.";
 
   LPDoc " -- Control --";
 
@@ -353,31 +360,15 @@ let core_builtins = let open BuiltIn in let open ContextualConversion in [
       { p = (<=); psym = "=<"; pname = "le_" } ;
       { p = (>=); psym = ">="; pname = "ge_" } ]
 
-  @ [
-
-  LPCode "type (<), (>), (=<), (>=) A -> A -> prop.";
-  LPCode "X  > Y  :- gt_ X Y.";
-  LPCode "X  < Y  :- lt_ X Y.";
-  LPCode "X  =< Y :- le_ X Y.";
-  LPCode "X  >= Y :- ge_ X Y.";
-
-  LPCode "type (i<), (i>), (i=<), (i>=) int -> int -> prop.";
-  LPCode "X i< Y  :- lt_ X Y.";
-  LPCode "X i> Y  :- gt_ X Y.";
-  LPCode "X i=< Y :- le_ X Y.";
-  LPCode "X i>= Y :- ge_ X Y.";
-
-  LPCode "type (r<), (r>), (r=<), (r>=) float -> float -> prop.";
-  LPCode "X r< Y  :- lt_ X Y.";
-  LPCode "X r> Y  :- gt_ X Y.";
-  LPCode "X r=< Y :- le_ X Y.";
-  LPCode "X r>= Y :- ge_ X Y.";
-
-  LPCode "type (s<), (s>), (s=<), (s>=) string -> string -> prop.";
-  LPCode "X s< Y  :- lt_ X Y.";
-  LPCode "X s> Y  :- gt_ X Y.";
-  LPCode "X s=< Y :- le_ X Y.";
-  LPCode "X s>= Y :- ge_ X Y.";
+  @ 
+  let build_symb (spref, ty) = 
+    let op_l = ["gt_";"lt_"; "le_"; "ge_"] in
+    let sym_l = List.map (fun x -> spref ^ x) [">";"<"; "=<"; ">="] in
+    let buildLPCode s op = LPCode (Printf.sprintf "pred (%s) i:%s, i:%s.\nX %s Y :- %s X Y." s ty ty s op) in
+    List.map2 buildLPCode sym_l op_l in
+  let symbs = ["", "A"; "i", "int"; "r", "float"; "s", "string"] in
+  List.flatten (List.map build_symb symbs) @
+  [
 
   LPDoc " -- Standard data types (supported in the FFI) --";
 
@@ -394,6 +385,21 @@ let core_builtins = let open BuiltIn in let open ContextualConversion in [
   LPCode "pred snd  i:pair A B, o:B.";
   LPCode "snd (pr _ B) B.";
 
+  LPCode {|
+kind triple type -> type -> type -> type.
+type triple A -> B -> C -> triple A B C.
+
+pred triple_1 i:triple A B C, o:A.
+triple_1 (triple A _ _) A.
+
+pred triple_2 i:triple A B C, o:B.
+triple_2 (triple _ B _) B.
+
+pred triple_3 i:triple A B C, o:C.
+triple_3 (triple _ _ C) C.
+ 
+|};
+
   MLData (option (BuiltInData.poly "A"));
 
   MLData cmp;
@@ -642,40 +648,9 @@ let lp_builtins = let open BuiltIn in let open BuiltInData in [
      | Sys_error msg -> error msg)),
   DocAbove);
 
-  LPDoc " -- Hacks --";
-
-  MLCode(Pred("string_to_term",
-    In(string, "S",
-    Out(any,   "T",
-    Full(ContextualConversion.unit_ctx, "parses a term T from S"))),
-  (fun text _ ~depth () () state ->
-     try
-       let state, t = Quotation.term_at ~depth state text in
-       state, !:t, []
-     with
-     | Parse.ParseError _ -> raise No_clause)),
-  DocAbove);
-
-  MLCode(Pred("readterm",
-    In(in_stream, "InStream",
-    Out(any,      "T",
-    Full(ContextualConversion.unit_ctx, "reads T from InStream, ends with \\n"))),
-  (fun (i,source_name) _ ~depth () () state ->
-     try
-       let text = input_line i in
-       let state, t = Quotation.term_at ~depth state text in
-       state, !:t, []
-     with
-     | Sys_error msg -> error msg
-     | Parse.ParseError _ -> raise No_clause)),
-  DocAbove);
-
   LPCode "pred printterm i:out_stream, i:A.";
   LPCode "printterm S T :- term_to_string T T1, output S T1.";
 
-  LPCode "pred read o:A.";
-  LPCode "read S :- flush std_out, input_line std_in X, string_to_term X S.";
-
   ]
 ;;
 
@@ -688,7 +663,7 @@ let elpi_builtins = let open BuiltIn in let open BuiltInData in let open Context
   MLCode(Pred("dprint",
     VariadicIn(unit_ctx, !> any, "prints raw terms (debugging)"),
   (fun args ~depth _ _ state ->
-     Format.fprintf Format.std_formatter "@[<hov 1>%a@]@\n%!"
+    Util.printf "@[<hov 1>%a@]@\n%!"
        (RawPp.list (RawPp.Debug.term depth) " ") args ;
      state, ())),
   DocAbove);
@@ -696,7 +671,7 @@ let elpi_builtins = let open BuiltIn in let open BuiltInData in let open Context
   MLCode(Pred("print",
     VariadicIn(unit_ctx, !> any,"prints terms"),
   (fun args ~depth _ _ state ->
-     Format.fprintf Format.std_formatter "@[<hov 1>%a@]@\n%!"
+     Util.printf "@[<hov 1>%a@]@\n%!"
        (RawPp.list (RawPp.term depth) " ") args ;
      state, ())),
   DocAbove);
@@ -705,32 +680,6 @@ let elpi_builtins = let open BuiltIn in let open BuiltInData in let open Context
 pred counter i:string, o:int.
 counter C N :- trace.counter C N.|};
 
-   MLCode(Pred("quote_syntax",
-     In(string, "FileName",
-     In(string, "QueryText",
-     Out(list (poly "A"), "QuotedProgram",
-     Out(poly "A",        "QuotedQuery",
-     Full    (unit_ctx, "quotes the program from FileName and the QueryText. "^
-              "See elpi-quoted_syntax.elpi for the syntax tree"))))),
-   (fun f s _ _ ~depth _ _ state ->
-      let elpi =
-        Setup.init
-          ~builtins:[BuiltIn.declare ~file_name:"(dummy)" []]
-          ~file_resolver:(Parse.std_resolver ~paths:[] ())
-          () in
-      try
-        let ap = Parse.program ~elpi ~files:[f] in
-        let loc = Ast.Loc.initial "(quote_syntax)" in
-        let aq = Parse.goal ~elpi ~loc ~text:s in
-        let p = Compile.(program ~flags:default_flags ~elpi [ap]) in
-        let q = API.Compile.query p aq in
-        let state, qp, qq = Quotation.quote_syntax_runtime state q in
-        state, !: qp +! qq, []
-      with Parse.ParseError (_,m) | Compile.CompileError (_,m) ->
-        Printf.eprintf "%s\n" m;
-        raise No_clause)),
-  DocAbove);
-
   MLData loc;
 
   MLCode(Pred("loc.fields",
@@ -797,15 +746,6 @@ rex_split Rx S L :- rex.split Rx S L.|};
 ;;
 
 (** ELPI specific NON-LOGICAL built-in *********************************** *)
-
-let ctype = AlgebraicData.declare {
-  AlgebraicData.ty = TyName "ctyp";
-  doc = "Opaque ML data types";
-  pp = (fun fmt cty -> Format.fprintf fmt "%s" cty);
-  constructors = [
-    K("ctype","",A(BuiltInData.string,N),B (fun x -> x), M (fun ~ok ~ko x -> ok x))  
-  ]
-} |> ContextualConversion.(!<)
    
 let safe = OpaqueData.declare {
   OpaqueData.name = "safe";
@@ -876,8 +816,6 @@ let elpi_nonlogical_builtins = let open BuiltIn in let open BuiltInData in let o
 
   LPDoc "== Elpi nonlogical builtins =====================================";
 
-  MLData ctype;
-
   MLCode(Pred("var",
     InOut(ioarg_any, "V",
     VariadicInOut(unit_ctx, !> (ioarg_any),"checks if the term V is a variable. When used with tree arguments it relates an applied variable with its head and argument list.")),
@@ -1017,8 +955,8 @@ X == Y :- same_term X Y.
 
   MLCode(Pred("is_cdata",
     In(any,     "T",
-    Out(ctype,  "Ctype",
-    Easy        "checks if T is primitive of type Ctype, eg (ctype \"int\")")),
+    Out(string,  "Ctype",
+    Easy        "checks if T is primitive of type Ctype, eg \"int\"")),
   (fun t _ ~depth ->
      match look ~depth t with
      | CData n -> !:(RawOpaqueData.name n)
@@ -1026,7 +964,7 @@ X == Y :- same_term X Y.
   DocAbove);
 
   LPCode "pred primitive? i:A, i:string.";
-  LPCode "primitive? X S :- is_cdata X (ctype S).";
+  LPCode "primitive? X S :- is_cdata X S.";
 
   MLCode(Pred("new_int",
      Out(int, "N",
@@ -1543,13 +1481,3 @@ let std_declarations =
 
 let std_builtins =
   BuiltIn.declare ~file_name:"builtin.elpi" std_declarations
-
-
-let default_checker () =
-  try
-    let elpi = API.Setup.init ~builtins:[std_builtins] () in
-    let ast = API.Parse.program_from ~elpi ~loc:(API.Ast.Loc.initial "(checker)") (Lexing.from_string Builtin_checker.code) in
-    API.Compile.program ~flags:API.Compile.default_flags ~elpi [ast]
-  with
-  | API.Parse.ParseError(loc,msg) -> API.Utils.anomaly ~loc msg
-  | API.Compile.CompileError(loc,msg) -> API.Utils.anomaly ?loc msg
