@@ -28,6 +28,9 @@ module Func : sig
   val arrowf : t
   val sequentf : t
   val ctypef : t
+  val propf : t
+  val typef : t
+  val mainf : t
 
   val dummyname : t
   val spillf : t
@@ -37,34 +40,14 @@ module Func : sig
   val from_string : string -> t
 
   module Map : Map.S with type key = t
+  module Set : Set.S with type elt = t
 end
 
-module Term : sig
+module Mode : sig
 
-  type t =
-   | Const of Func.t
-   | App of t * t list
-   | Lam of Func.t * t
-   | CData of CData.t
-   | Quoted of quote
-  and quote = { data : string; loc : Loc.t; kind : string option }
-  [@@ deriving show, ord]
+  type t = Input | Output
+  [@@deriving show, ord]
 
-  exception NotInProlog of Loc.t * string
-
-  (* Can raise NotInProlog *)
-  val mkApp : Loc.t -> t list -> t
-
-  val mkAppF : Loc.t -> Func.t -> t list -> t
-
-  val mkCon : string -> t
-  val mkNil : t
-  val mkSeq : t list -> t
-  val mkQuoted : Loc.t -> string -> t
-  val mkFreshUVar : unit -> t
-  val mkFreshName : unit -> t
-  val mkLam : string -> t -> t
-  val mkC : CData.t -> t
 end
 
 type raw_attribute =
@@ -73,16 +56,67 @@ type raw_attribute =
   | After of string
   | Before of string
   | Replace of string
+  | Remove of string
   | External
   | Index of int list * string option
+  | Functional
+  | Untyped
 [@@ deriving show]
 
+module TypeExpression : sig
+
+  type 'attribute t_ =
+   | TConst of Func.t
+   | TApp of Func.t * 'attribute t * 'attribute t list
+   | TPred of 'attribute * (Mode.t * 'attribute t) list
+   | TArr of 'attribute t * 'attribute t
+   and 'a t = { tit : 'a t_; tloc : Loc.t }
+   [@@ deriving show, ord]
+
+end
+
+module Term : sig
+  type typ = raw_attribute list TypeExpression.t
+  [@@ deriving show, ord]
+  type t_ =
+   | Const of Func.t
+   | App of t * t list
+   | Lam of Func.t * typ option * t
+   | CData of CData.t
+   | Quoted of quote
+   | Cast of t * typ
+   | Parens of t
+  and t = { it : t_; loc : Loc.t }
+  and quote = { qloc : Loc.t; data : string; kind : string option }
+  [@@ deriving show, ord]
+
+  exception NotInProlog of Loc.t * string
+
+  (* Can raise NotInProlog *)
+  val mkApp : Loc.t -> t list -> t
+  val mkParens : Loc.t -> t -> t
+  val mkAppF : Loc.t -> (Loc.t * Func.t) -> t list -> t
+
+  val mkCon : Loc.t -> string -> t
+  val mkConst : Loc.t -> Func.t -> t
+  val mkNil : Loc.t -> t
+  val mkSeq : ?loc:Loc.t -> t list -> t
+  val mkQuoted : Loc.t -> int -> string -> t
+  val mkFreshUVar : Loc.t -> t
+  val mkFreshName : Loc.t -> t
+  val mkLam : Loc.t -> string -> typ option -> t -> t
+  val mkC : Loc.t -> CData.t -> t
+  val mkCast : Loc.t -> t -> typ -> t
+
+end
+
 module Clause : sig
 
-  type ('term,'attributes) t = {
+  type ('term,'attributes,'spill) t = {
     loc : Loc.t;
     attributes : 'attributes;
     body : 'term;
+    needs_spilling : 'spill;
   }
   [@@ deriving show, ord]
 
@@ -90,16 +124,16 @@ end
 
 module Chr : sig
 
-  type sequent = { eigen : Term.t; context : Term.t; conclusion : Term.t }
-  and 'attribute t = {
-    to_match : sequent list;
-    to_remove : sequent list;
-    guard : Term.t option;
-    new_goal : sequent option;
+  type 'term sequent = { eigen : 'term; context : 'term; conclusion : 'term }
+  and ('attribute,'term) t = {
+    to_match : 'term sequent list;
+    to_remove : 'term sequent list;
+    guard : 'term option;
+    new_goal : 'term sequent option;
     attributes : 'attribute;
     loc : Loc.t;
   }
-  [@@ deriving show]
+  [@@ deriving show,ord]
 
 end
 
@@ -115,29 +149,26 @@ end
 
 module Type : sig
 
-  type 'attribute t = {
+  type ('attribute,'inner_attribute) t = {
     loc : Loc.t;
     attributes : 'attribute;
     name : Func.t;
-    ty : Term.t;
+    ty : 'inner_attribute TypeExpression.t;
   }
-  [@@ deriving show]
-
-end
-
-module Mode : sig
-
-  type 'name t =
-    { name : 'name; args : bool list; loc : Loc.t }
-  [@@ deriving show, ord]
+  [@@deriving show, ord]
 
 end
 
 module TypeAbbreviation : sig
 
-  type ('name) t =
-    { name : 'name; value : Term.t; nparams : int; loc : Loc.t }
-  [@@ deriving show]
+  type 'ty closedTypeexpression = 
+    | Lam of Func.t * Loc.t * 'ty closedTypeexpression 
+    | Ty of 'ty
+  [@@ deriving show, ord]
+
+  type ('name,'ty) t =
+    { name : 'name; value : 'ty closedTypeexpression; nparams : int; loc : Loc.t }
+  [@@ deriving show, ord]
 
 end
 
@@ -151,22 +182,20 @@ module Program : sig
     | Shorten of Loc.t * (Func.t * Func.t) list (* prefix suffix *)
     | End of Loc.t
 
-    | Accumulated of Loc.t * (Digest.t * decl list) list
+    | Accumulated of Loc.t * parser_output list
 
     (* data *)
-    | Clause of (Term.t, raw_attribute list) Clause.t
-    | Local of Func.t list
-    | Mode of Func.t Mode.t list
-    | Chr of raw_attribute list Chr.t
+    | Clause of (Term.t, raw_attribute list, unit) Clause.t
+    | Chr of (raw_attribute list,Term.t) Chr.t
     | Macro of (Func.t, Term.t) Macro.t
-    | Type of raw_attribute list Type.t list
-    | Pred of raw_attribute list Type.t * Func.t Mode.t
-    | TypeAbbreviation of Func.t TypeAbbreviation.t
+    | Kind of (raw_attribute list,raw_attribute list) Type.t list
+    | Type of (raw_attribute list,raw_attribute list) Type.t list
+    | Pred of (raw_attribute list,raw_attribute list) Type.t
+    | TypeAbbreviation of (Func.t,raw_attribute list TypeExpression.t) TypeAbbreviation.t
     | Ignored of Loc.t
+  and parser_output = { file_name : string; digest : Digest.t; ast : decl list }
   [@@ deriving show]
 
-  val mkLocal : string list -> decl
-
   type t = decl list
   [@@ deriving show]
 
@@ -174,7 +203,7 @@ end
 
 module Goal : sig
 
-  type t = Loc.t * Term.t
+  type t = Term.t
   [@@ deriving show]
 
 end
@@ -192,28 +221,31 @@ module Structured : sig
 
 type program = {
   macros : (Func.t, Term.t) Macro.t list;
-  types : tattribute Type.t list;
-  type_abbrevs : Func.t TypeAbbreviation.t list;
-  modes : Func.t Mode.t list;
+  kinds : (unit,unit) Type.t list;
+  types : (tattribute,functionality) Type.t list;
+  type_abbrevs : (Func.t,functionality TypeExpression.t) TypeAbbreviation.t list;
+  modes : (tattribute,functionality) Type.t list;
   body : block list;
 }
-and block_constraint = {
-   clique : Func.t list;
-   ctx_filter : Func.t list;
-   rules : cattribute Chr.t list
+and ('func,'term) block_constraint = {
+   clique : 'func list;
+   ctx_filter : 'func list;
+   rules : (cattribute,'term) Chr.t list
 }
 and block =
-  | Locals of Func.t list * program
-  | Clauses of (Term.t,attribute) Clause.t list
+  | Clauses of (Term.t,attribute,unit) Clause.t list
   | Namespace of Func.t * program
   | Shorten of Func.t shorthand list * program
-  | Constraints of block_constraint * program
+  | Constraints of (Func.t,Term.t) block_constraint * program
+  | Accumulated of program
 and attribute = {
   insertion : insertion option;
   id : string option;
   ifexpr : string option;
+  typecheck : bool;
 }
-and insertion = Before of string | After of string | Replace of string
+and insertion = Insert of insertion_place | Replace of string | Remove of string
+and insertion_place = Before of string | After of string
 and cattribute = {
   cid : string;
   cifexpr : string option
@@ -227,6 +259,8 @@ and 'a shorthand = {
   full_name : 'a;
   short_name : 'a;
 }
+and functionality = Function | Relation
+and variadic = Variadic | NotVariadic
 [@@deriving show, ord]
 
 end
