@@ -10,6 +10,7 @@ type expectation =
   | Failure
   | SuccessOutput of Str.regexp
   | FailureOutput of Str.regexp
+  | SuccessOutputTxt of (string list -> bool)
   | SuccessOutputFile of { sample : string; adjust : string -> string; reference : string }
 
 type trace = Off | On of string list
@@ -24,12 +25,10 @@ type t = {
   source_dune : fname option;
   source_json : fname option;
   after: string list;
-  typecheck : bool;
   input: fname option;
   expectation : expectation;
   outside_llam : bool;
   trace : string list;
-  legacy_parser : bool;
 }
 
 let tests = ref []
@@ -37,16 +36,17 @@ let tests = ref []
 let declare
     name ~description ?source_elpi ?source_teyjus ?(deps_teyjus=[]) ?source_dune ?source_json
     ?after
-    ?(typecheck=true) ?input ?(expectation=Success)
+    ?input ?(expectation=Success)
     ?(outside_llam=false)
     ?(trace=Off)
-    ?(legacy_parser=false)
     ?(promote=false)
     ~category
     ()
 =
   if List.exists (fun { name = x; _ } -> x = name) !tests then
     failwith ("a test named " ^ name ^ " already exists");
+  if String.index_opt name ' ' <> None then
+    failwith ("test name '" ^ name ^ "' contains invalid character");
   begin match source_elpi, source_teyjus, source_dune, source_json with
     | None, None, None, None-> failwith ("test "^name^" has no sources");
     | _ -> ()
@@ -60,12 +60,10 @@ let declare
     source_dune;
     source_json;
     after = (match after with None -> [] | Some x -> [x]);
-    typecheck;
     input;
     expectation;
     category;
     outside_llam;
-    legacy_parser;
     trace = (match trace with Off -> [] | On l -> "-trace-on" :: l)
   } :: !tests
 
@@ -274,6 +272,20 @@ let with_log (_,log) f =
     x
   with e -> close_in cin; raise e
 
+let read_lines fname =
+  let cin = open_in fname in
+  let rec aux () =
+    try
+      let x = input_line cin in
+      x :: aux ()
+    with End_of_file -> close_in cin; []
+  in
+  aux ()
+
+let has_substring ~sub s = 
+  try Str.(search_forward (regexp sub)) s 0 |> ignore; true
+  with Not_found -> false
+
 let option_map f = function None -> None | Some x -> Some (f x)
 
 let strip_cwd file =
@@ -361,20 +373,9 @@ let read_tctime input_line =
   done; !time
   with End_of_file -> !time
 
-let legacy_parser_available executable =
-  let log = Util.open_dummy_log () in
-  let env = Unix.environment () in
-  match
-    Util.exec ~executable ~timeout:1.0 ~env ~log ~args:["-legacy-parser-available"] ()
-  with
-  | Util.Exit(0,_,_) -> true
-  | _ -> false
-
 let () = Runner.declare
-  ~applicable:begin fun ~executable { Test.source_elpi; legacy_parser; _ } ->
-    if is_elpi executable && source_elpi <> None && 
-      (not legacy_parser || legacy_parser_available executable)
-      then Runner.Can_run_it
+  ~applicable:begin fun ~executable { Test.source_elpi; _ } ->
+    if is_elpi executable && source_elpi <> None then Runner.Can_run_it
     else Runner.Not_for_me
   end
   ~run:begin fun ~executable ~timetool ~timeout ~env ~sources ~promote test ->
@@ -387,15 +388,9 @@ let () = Runner.declare
     Util.write log (Printf.sprintf "executable: %s\n" executable);
     let executable_stuff = Filename.dirname executable ^ "/../lib/elpi/" in
 
-    let { Test.expectation; input; outside_llam ; typecheck; trace; legacy_parser; _ } = test in
+    let { Test.expectation; input; outside_llam ; trace; _ } = test in
     let input = Util.option_map (fun x -> sources^x) input in
     let args = ["-test";"-I";executable_stuff;"-I";sources;source] @ trace in
-    let args =
-      if typecheck then args
-      else "-no-tc" :: args in
-    let args =
-      if not legacy_parser then args
-      else "-legacy-parser" :: args in
     let args =
       if outside_llam then "-delay-problems-outside-pattern-fragment"::args
       else args in
@@ -415,6 +410,7 @@ let () = Runner.declare
         | Test.SuccessOutputFile { sample; adjust; reference } when promote ->
             FileUtil.cp [adjust sample] (sources^"/"^reference);
             Runner.Promote { walltime; typechecking; execution; mem }
+        | Test.SuccessOutputTxt f when f (Util.read_lines (snd log)) -> Runner.Success { walltime; typechecking; execution; mem }
         | _ -> Runner.Failure { walltime; typechecking; execution; mem }
         end
 
@@ -479,6 +475,8 @@ module ElpiTraceElab = struct
       let execution = 0.0 in
       let rc =
         match outcome, outcomey, test.Test.expectation with
+        | Util.Exit(0,walltime,mem), Some(Util.Exit(0,_,_)), Test.SuccessOutputTxt f when f (Util.read_lines (snd log)) -> 
+            Runner.Success { walltime; typechecking; execution; mem }
         | Util.Exit(0,walltime,mem), Some(Util.Exit(0,_,_)), Test.SuccessOutputFile { sample; adjust; reference } when match_file ~log sample adjust (sources^"/"^reference) ->
             Runner.Success { walltime; typechecking; execution; mem }
         | Util.Exit(0,walltime,mem), Some(Util.Exit(0,_,_)), Test.SuccessOutputFile { sample; adjust; reference } when promote ->
@@ -572,30 +570,21 @@ end
 
 module Dune = struct
 
-let is_dune =
-  let rex = Str.regexp "dune" in
-  fun s -> Str.string_match rex (Filename.basename s) 0
-
-let is_dune_src = function
-  | None -> false
-  | Some s -> Filename.check_suffix s ".ml"
+let same_binary exe pat =
+  match pat with None -> false
+  | Some pat -> Str.string_match (Str.regexp (".*"^pat)) exe 0
 
 let () = Runner.declare
   ~applicable:begin fun ~executable { source_dune; _ } ->
-    if is_dune executable && is_dune_src source_dune then Runner.Can_run_it else Runner.Not_for_me
+    if same_binary executable source_dune then Runner.Can_run_it else Runner.Not_for_me
   end
   ~run:begin fun ~executable ~timetool ~timeout ~env ~sources ~promote test ->
-  let source =
-    match test.Test.source_dune with Some x -> x | _ -> assert false in
   if not (Sys.file_exists executable) then Runner.Skipped
-  else if not (Sys.file_exists (sources^source)) then Runner.Skipped
   else
-    let log = Util.open_log ~executable test in
+    let log = Util.open_log ~executable:(Filename.basename executable) test in
+    let { Test.expectation; input; outside_llam = _; _ } = test in
     Util.write log (Printf.sprintf "executable: %s\n" executable);
-    let { Test.expectation; input; outside_llam = _ ; typecheck = _; _ } = test in
-    let sources = Str.global_replace (Str.regexp "^.*tests/sources/") "tests/sources/" sources in
-    let source = Filename.remove_extension source ^ ".exe" in
-    let args = ["exec"; sources ^ "/" ^ source; "--"; "-I"; "src/"] in
+    let args = ["-I"; "src/"] in
     Util.write log (Printf.sprintf "args: %s\n" (String.concat " " args));
     let rc =
       let outcome = Util.exec ~timeout ~timetool ?input ~executable ~env ~log ~args () in
